/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.authentication;

import java.util.Vector;

/** 
 * This interface provides an authentication mechanism by
 * using the "WWW-Authenticate" and the "Authorization"
 * HTTP headers.
 *
 * @author		Marc Eaddy
 * @version		1.0
 * @date        2 May 1998
 */

public interface AuthenticationInterface
{
	/**
	 * 
	 *
	 * @param strAuthorizationHeader    [IN] The "Authorization" header
     *                                  from the Http request.
     * @param strRealm                  [IN] The realm.
     * @param strMethod                 [IN] The HTTP/WebDAV request
     *                                  method.
     * @param strRequestUri             [IN] The resource URI from the
     *                                  HTTP request.
	 * @param vecWWWAuthenticateHeaders [OUT] On AUTH_FAILURE, will
     *                                  contain 1 or more
     *                                  WWWAuthenticateHeaders which
     *                                  should be sent to the client
     *                                  in the response.
	 *
	 * @return		One of the following status codes:<P>
	 *
     * <UL>
	 *		<LI>AUTH_SUCCESS            The client was successfully
     *                                  authenticated.
	 *		<LI>AUTH_INTERNAL_ERROR     There was an internal error.
	 *		<LI>AUTH_FAILURE            The client either submitted
     *                                  invalid authorization or no
     *                                  authorization at all.  The
     *                                  vecWWWAuthenticateHeaders
     *                                  will be filled with 1 or more
     *                                  WWWAuthenticateHeader objects
     *                                  which should be sent to the
     *                                  client in the response.
     *      <LI>AUTH_BAD_REQUEST        The authorization header was
     *                                  invalid.
	 * </UL>
	 */
	public int authenticate(String strAuthorizationHeader,
                            String strRemoteIPAddress,
                            String strRealm,
                            String strMethod,
                            String strRequestUri,
                            Vector vecWWWAuthenticateHeaders);

	/** Success */
	public final static int AUTH_SUCCESS				= 0;

	/** Unknown or unhandled failure */
	public final static int AUTH_FAILURE				= -3000;

	/** Internal error */
	public final static int AUTH_INTERNAL_ERROR		    = -3001;

	/** The Authorization request was invalid */
	public final static int AUTH_BAD_REQUEST			= -3002;
}
