/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED

  Permission to use, copy, modify, and distribute this software and its
  documentation for research and educational purpose and without fee is
  hereby granted, provided that the above copyright notice appear in all
  copies and that both that the copyright notice and warranty disclaimer
  appear in supporting documentation, and that the names of the
  copyright holders or any of their entities not be used in advertising
  or publicity pertaining to distribution of the software without
  specific, written prior permission.  Use of this software in whole or
  in parts for direct commercial advantage requires explicit prior
  permission.
  
	The copyright holders disclaim all warranties with regard to this
	software, including all implied warranties of merchantability and
	fitness.  In no event shall the copyright holders be liable for any
	special, indirect or consequential damages or any damages whatsoever
	resulting from loss of use, data or profits, whether in an action of
	contract, negligence or other tortuous action, arising out of or in
	connection with the use or performance of this software.
*/

package webdav.client;

import java.awt.*;
import java.util.*;

/**
* This class is gui part of webdav client, it
* compose a frame for user to enter request
* @author Shao Rong
* @version 1.0
*
*/

public class ClientFrame 
    extends Frame 
    implements WebdavMethod 
{
	public final static int MaxFieldNum = 12; // must be even
	
	Button buttonOk = new Button ("OK");
	Button buttonClear = new Button ("Clear");
	Button buttonCancel = new Button ("Close");
	
	Label labelMethod = new Label ("Method : ");
	Choice choiceMethod  = new Choice ();
	
	Label labelHostName = new Label ("Hostname : ");
	TextField textHostName = new TextField ("localhost", 40);
	
	Label labelPortNumber = new Label ("Port Number : ");
	TextField textPortNumber = new TextField ("8080", 15);
	
	Label labelUserName = new Label("User Name: ");
	TextField textUserName = new TextField("", 15);
	
    Label labelPassword = new Label("Password: ");
    TextField textPassword = new TextField("", 15);

	Label labelResource = new Label ("Resource name : ");
	TextField textResource = new TextField ("/index.html", 40);
	
	TextField textFields[] = new TextField [MaxFieldNum];
	
	TextArea textResponse = new TextArea (16,40);
	
	/** 
	 * @param hdrNames : header names 
	 * @param hdrValues : header values
	 */
	public ClientFrame (Vector hdrNames, Vector hdrValues) 
    {
        for (int i=0; i < MaxFieldNum; ++i) 
        {
			if (i % 2 == 0) 
				textFields[i] = new TextField ("", 10);
			else  textFields[i] = new TextField ("", 40);
		}
		
		emptyFields ();
		fillFields (hdrNames, hdrValues);
		addObj();
		Dimension d = new Dimension (600,600);
		setSize (d);
		show ();
	}
	
	public ClientFrame () 
    {
		for (int i=0; i<MaxFieldNum; ++i) 
        {
			if (i % 2 == 0) 
				textFields[i] = new TextField ("", 10);
			else
				textFields[i] = new TextField ("", 40);
		}
		
		emptyFields ();
		addObj();
		Dimension d = new Dimension (600,600);
		setSize (d);
		setTitle("WebDAV Client");
		show ();
	}
	
	//*-------------------------------------------------------------- 	
	
	/** 
	 * 
	 * add gui needed componets
	 */
	public void addObj () 
    {
		Panel panelMethod = new Panel();
		panelMethod.setLayout (new FlowLayout (FlowLayout.LEFT));
		panelMethod.add (labelMethod);
		
		choiceMethod.addItem (strGetMethod);
		choiceMethod.addItem (strPutMethod);
		choiceMethod.addItem (strDeleteMethod);
		choiceMethod.addItem (strMoveMethod);
		choiceMethod.addItem (strCopyMethod);
		choiceMethod.addItem (strPropFindMethod);
		choiceMethod.addItem (strPropPatchMethod);
		choiceMethod.addItem (strLockMethod);
		choiceMethod.addItem (strUnlockMethod);
		choiceMethod.addItem (strMkcolMethod);
		
		panelMethod.add (choiceMethod);
		
		Panel panelHostName = new Panel();
		panelHostName.setLayout (new FlowLayout (FlowLayout.LEFT));
		panelHostName.add (labelHostName);
		panelHostName.add (textHostName);
		
		Panel panelPortNum = new Panel();
		panelPortNum.setLayout (new FlowLayout (FlowLayout.LEFT));
		panelPortNum.add (labelPortNumber);
		panelPortNum.add (textPortNumber);
		
		Panel panelUserName = new Panel();
		panelUserName.setLayout (new FlowLayout (FlowLayout.LEFT));
		panelUserName.add(labelUserName);
		panelUserName.add(textUserName);

        Panel panelPassword = new Panel();
        panelPassword.setLayout (new FlowLayout (FlowLayout.LEFT));
        panelPassword.add(labelPassword);
        panelPassword.add(textPassword);
        textPassword.setEchoCharacter('*');

		Panel panelResource = new Panel();
		panelResource.setLayout (new FlowLayout (FlowLayout.LEFT));
		panelResource.add (labelResource);
		panelResource.add (textResource);
		
		Panel aryHeaderPanels[] = new Panel[MaxFieldNum/2];
		
		for (int i=0; i < MaxFieldNum/2; ++i) 
        {
			aryHeaderPanels[i] = new Panel();
			aryHeaderPanels[i].setLayout (new FlowLayout (FlowLayout.LEFT));
			aryHeaderPanels[i].add (textFields[i*2]);
			aryHeaderPanels[i].add (textFields[i*2+1]);
		}
		
		Panel panelCenter = new Panel();
		panelCenter.setLayout (new GridLayout(MaxFieldNum/2+6, 1));
		panelCenter.add (panelMethod);
		panelCenter.add (panelHostName);
		panelCenter.add (panelPortNum);
		panelCenter.add (panelUserName);
        panelCenter.add (panelPassword);
		panelCenter.add (panelResource);
		
		for (int i=0; i < MaxFieldNum/2; ++i) 
        {
			panelCenter.add(aryHeaderPanels[i]); 
		}
		
		Panel panelSouth = new Panel();
		panelSouth.setLayout (new FlowLayout (FlowLayout.RIGHT));
		panelSouth.add(buttonOk); 
		panelSouth.add(buttonClear); 
		panelSouth.add(buttonCancel); 
		
		Panel p2 = new Panel ();
		p2.setLayout (new BorderLayout());
		p2.add ("Center", panelCenter);
		p2.add ("South", panelSouth);
		
		Panel panelWhole = new Panel ();
		panelWhole.setLayout (new BorderLayout());
		panelWhole.add ("Center", p2);
		
		textResponse.setText ("");
		
		panelWhole.add ("South", textResponse);
		
		add (panelWhole);
	}

	/** 
	 * 
	 * handle Events triggered by component in the frame
	 */
	public boolean handleEvent(Event event) 
    {
		if (event.id == java.awt.Event.ACTION_EVENT) 
        {
			if (event.target instanceof Button) 
            {
				Button buttonChoice = (Button) event.target;
				if (buttonChoice.getLabel().equals("OK")) 
                {
					// build an vector and pass to the webdavConnection
					// using a vector to do it
					
					Vector vecHeaderNames = new Vector ();
					Vector vecHeaderValues = new Vector ();
					
					int i=0;
					
					while (i < MaxFieldNum/2 && 
                          !textFields[i*2].getText().equals("")) 
                    {
						vecHeaderNames.addElement (textFields[i*2].getText());
						vecHeaderValues.addElement (textFields[i*2+1].getText());
						++i;
					}
					
					WebdavToHttpRequest webdavRequest = 
						new WebdavToHttpRequest (	
                                            choiceMethod.getSelectedItem(),
											textHostName.getText(),
											Integer.parseInt (textPortNumber.getText()),
											textUserName.getText(),
                                            textPassword.getText(),
											textResource.getText(),
											vecHeaderNames,
											vecHeaderValues
											);
					webdavRequest.sendRequest ();
					textResponse.setText(webdavRequest.procRsp());
					
					return true;
				}
				else if (buttonChoice.getLabel().equals("Clear")) 
                {
					emptyFields ();
					return true;
				}
				else if (buttonChoice.getLabel().equals("Close")) 
                {
					this.dispose ();
					System.exit (0);
				}
			}
			else if (event.target instanceof Choice) 
            {
				Choice c = (Choice)event.target;
				
				String s = new String (c.getSelectedItem ());
				
				this.emptyFields ();
				
				Vector hdrNames = new Vector ();
				Vector hdrValues = new Vector ();
				
				WebdavToHttpRequest.getHeaders (s, hdrNames, hdrValues);
				
				this.fillFields (hdrNames, hdrValues);
				
				return true;
			}
		}
		
		return super.handleEvent(event);
	}

	/** 
	 * 
	 * empty all fields in the frame
	 */
	public void emptyFields () 
    {
		for (int i=0; i < MaxFieldNum; ++i) 
        {
			textFields[i].setText (""); 
		}
		
		textResponse.setText("");
	}

	/**  
	 * fill frame with headernames and headervalues
	 * 
	 * @param hdrNames : header names
	 * @param hdrValues : header values
	 */
	public void fillFields (Vector hdrNames, Vector hdrValues) 
    {
		if (hdrNames.size() != hdrValues.size()) 
        {
			System.out.println ("header names and header values should be the same count");
			System.exit (-1);
		}
		else if (hdrNames.size() > MaxFieldNum) 
        {
			System.out.println ("too many headers, and max headers is "+ MaxFieldNum);
			System.exit (-1);
		}
		
		Enumeration hdrs = hdrNames.elements();
		Enumeration hdrVals = hdrValues.elements ();
		
		int i=0;
		
		while (hdrs.hasMoreElements()) 
        {
			textFields[i*2].setText ((String)hdrs.nextElement());
			textFields[i*2+1].setText ((String)hdrVals.nextElement());
			++i;
		}
	}

	public final static void main (String args[]) 
    {
		ClientFrame frameClient = new ClientFrame ();
	}
}
