/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED

  Permission to use, copy, modify, and distribute this software and its
  documentation for research and educational purpose and without fee is
  hereby granted, provided that the above copyright notice appear in all
  copies and that both that the copyright notice and warranty disclaimer
  appear in supporting documentation, and that the names of the
  copyright holders or any of their entities not be used in advertising
  or publicity pertaining to distribution of the software without
  specific, written prior permission.  Use of this software in whole or
  in parts for direct commercial advantage requires explicit prior
  permission.
  
	The copyright holders disclaim all warranties with regard to this
	software, including all implied warranties of merchantability and
	fitness.  In no event shall the copyright holders be liable for any
	special, indirect or consequential damages or any damages whatsoever
	resulting from loss of use, data or profits, whether in an action of
	contract, negligence or other tortuous action, arising out of or in
	connection with the use or performance of this software.
*/

package webdav.client;

import java.util.*;
import java.io.*;
import HTTPClient.*;

/**
* This class is the gateway between the gui and the httpconnection package.
* It assembles the input data from the gui then use httpconnection to send
* a request.  Then it feeds the gui with the response information
* @author Shao Rong
* @version 1.0
*
*/

public class WebdavToHttpRequest
{
	public final static String s_strStateTokenPostfix = ".webdavltoken";
	public final static String s_strWebdavPathName = "WebdavPath";

	String          m_strWebdavPath     = new String ("");
        // webdav path where file written and read    

	boolean         m_bErrorFlag        = false;
    boolean         m_bDebug            = false;
	String          m_strMethod         = null;
	String          m_strResource       = null;
	NVPair          m_aryHeaderPairs[]  = null;
	byte            m_buf[]             = null;
	String          m_strHostName       = null;
	String          m_strUserName       = null;
	int             m_nPort             = -1;
	Vector          m_vecHeaderNames    = null;
	Vector          m_vecHeaderValues   = null;
	HTTPConnection  m_httpConnection    = null;
	HTTPResponse    m_httpResponse      = null;
	
	/**
	 * @param strMethod : name of the method
	 * @param strHost : host name
	 * @param nPort : port to attach to
	 * @param strResourceUri : the resource name
	 * @param vecHeaderNames : header names
	 * @param vecHeaderValues : header values
	 *
	 */
	WebdavToHttpRequest(	String strMethod, 
					String strHost, 
					int nPort, 
					String strUsername, 
                    String strPassword,
					String strResourceUri,
					Vector vecHeaderNames, 
					Vector vecHeaderValues) 
    {
		m_strHostName   = new String (strHost);  
		m_strMethod     = new String (strMethod);
		m_nPort         = nPort;
		m_strUserName   = new String(strUsername);
		
		// put a "/" in front if not
		
		if (strResourceUri.indexOf("/") != 0) 
        {
			m_strResource = new String ("/" + strResourceUri);
		}
		else 
        {
			m_strResource = new String (strResourceUri);
		}
		
		m_httpConnection = new HTTPConnection (strHost, nPort, 
                                                strUsername, strPassword);
		
		if (vecHeaderNames.size() != vecHeaderValues.size()) 
        {
			System.err.print("Header names size "+vecHeaderNames.size() + " is not equal to ");
			System.err.println ("header values size " + vecHeaderValues.size());
			System.exit (0);
		}
		
		m_vecHeaderNames = vecHeaderNames;
		m_vecHeaderValues = vecHeaderValues;
		
        getWebdavPath ();
	}
	
	/**
	 * Get the path of webdav where all files are put and read
	 */ 
	void getWebdavPath () 
    {
		String strProperty;
		strProperty = System.getProperty(s_strWebdavPathName);
		if (strProperty != null) 
        {
			m_strWebdavPath = new String (strProperty);
			File file = new File (strProperty);
			if (!file.exists() && !make_dirs (file.toString())) 
            {
				System.err.println ("Error in creating Webdav direcotry " + file);
				System.exit (-1);
			}
		}
		else 
        {
			m_strWebdavPath = new String (".");
		}
		// if (m_bDebug) System.err.println ("Webdav direcotry is : " + m_strWebdavPath);
	}
	
	
	/**
	 * Get the lock token of resource
	 * @param  strFileName : the name of resource
	 * @return lock token 
	 */ 
	String getStateToken (String strFileName) 
    {
		String strStateToken = new String ("");
		String strTokenFileName = new String (m_strWebdavPath +strFileName + 
                                            s_strStateTokenPostfix);
		try 
        {
			FileInputStream fileIn = new FileInputStream (strTokenFileName);
			DataInputStream dataIn = new DataInputStream (fileIn);
			strStateToken = new String (dataIn.readLine ());
			fileIn.close();
		}
		catch (FileNotFoundException e) 
        {
			// if (m_bDebug) System.err.println (strTokenFileName + " not found");
		}
		catch (IOException e) 
        {
			if (m_bDebug) System.err.println ("Error reading " + strTokenFileName);
		}
		catch (Exception e) 
        {
			if (m_bDebug) System.err.println ("Error reading " + strTokenFileName);
		}
		
		return strStateToken;
	}
	
	
	/**
	* fill headers of all request
	*/ 
	void fillHeaders () 
    {
		String strDestination = new String ("");
		
		Enumeration enumNames = m_vecHeaderNames.elements ();
		Enumeration enumValues = m_vecHeaderValues.elements ();
		
		int nLength = m_vecHeaderNames.size() + 2;
		
		m_aryHeaderPairs = new NVPair [nLength];
		
		int i = 0;
		
		while (enumNames.hasMoreElements()) 
        {
			String strName = new String ((String)enumNames.nextElement());
			String strValue = new String ((String)enumValues.nextElement());
			
			// need to make sure http://hostname/ is in front ??
			
			if (strName.equals("Destination")) 
            {
				strDestination = new String (strValue);
			}
			
			m_aryHeaderPairs[i++] = new NVPair (strName, strValue);
		}
		
		m_aryHeaderPairs[i++] = new NVPair("Owner-Info", m_strUserName);
		
		// for copy_method, only the resource's lock token is sent
		
		if (m_strMethod.equals (WebdavMethod.strCopyMethod)) 
        {
			m_aryHeaderPairs[i++] = new NVPair ("Lock-Token", 
                                                getStateToken (strDestination));
		}
		else 
        {
			m_aryHeaderPairs[i++] = new NVPair ("Lock-Token", 
                                                getStateToken (m_strResource));
		}
	}
	
	/**
	 * obtain the entity body(data) for the request
	 * @param resource : resource name
	 */ 
	void getData (String strResourceUri) 
    {
		String strTemp = new String (m_strWebdavPath+strResourceUri);
		// to get rid of "\" or "/" problem
		String strFileName = new String (strTemp.replace ('/', File.separatorChar));
		
		try 
        {
			File file = new File (strFileName); 
			
			if (!file.isFile ()) 
            {
				m_bErrorFlag = true;
				if (m_bDebug) System.err.println (strFileName + " is not a file or may not exist");
				return;
			}
			
			m_buf = new byte[(int)file.length()]; // a hack
			
			FileInputStream inStream = new FileInputStream (file);
			
			inStream.read (m_buf);
			inStream.close();
		}
		catch (Exception e) 
        {
			if (m_bDebug) System.err.println ("Error reading files " + strFileName);
			m_bErrorFlag = true;
		}
	}
	
	// detach the data from response to a file
	
	/**
	* detach the data from the response and write to a file
	* @param strResourceUri : resource name
	*/ 
	void putData (String strResourceUri) 
    {
		String strTemp = new String (m_strWebdavPath+strResourceUri);

        // to get rid of "\" or "/" problem
		String strFileName = new String (strTemp.replace ('/', File.separatorChar));
		
		if (!make_dirs (strFileName)) 
        {
			System.out.println ("Error creating all necessary parent dirs for file " + strFileName);
			m_bErrorFlag = true;
			return;
		}
		
		FileOutputStream outFile = null;
		
		try 
        {
			outFile = new FileOutputStream (strFileName);
		}
		catch (IOException e) 
        {
			m_bErrorFlag = true;
			System.err.println ("Error opening " + strFileName);
			return;
		}
		
		try 
        {
			if (outFile != null) 
            {
				outFile.write (m_httpResponse.getData());
				outFile.close();
			}
		}
		catch (IOException e) 
        {
			if (m_bDebug) System.err.println ("IOException in writing " + strFileName);
			m_bErrorFlag = true;
		}
	}
	
	/**
	 * make directories 
	 * @param fName_in : path name
	 */ 
	boolean make_dirs (String fName_in) 
    {
		String strFileName = new String (fName_in.replace ('/', File.separatorChar));
		File file  = null;
		Vector vecFileDirs = new Vector ();
		
		try 
        {
			File fileTemp = null;
			String strDir = new String ("");
			
			file = new File (strFileName);
			fileTemp = new File (strFileName);
			
			// get all parents dirs
			do 
            {
				strDir = fileTemp.getParent ();
				
				if (strDir != null && !strDir.equals(m_strWebdavPath)) 
                {
					fileTemp = new File (strDir);
					vecFileDirs.addElement (fileTemp);
				}
				
			}
			while ((strDir != null) && (!strDir.equals(m_strWebdavPath)));
			
			// make all necessary directory
			int vecSize = vecFileDirs.size();
			for (int i=(vecSize-1); i>=0; --i) 
            {
				fileTemp = (File) vecFileDirs.elementAt (i);
                
                // webdav path where file written and read    
				if (!fileTemp.exists () && !fileTemp.mkdir ()) 
                {
					if (m_bDebug) System.err.println ("failure to make the directory "+fileTemp);
					return false;
				}
				else if (fileTemp.isFile()) 
                {
					if (m_bDebug) System.err.println ("Failure to make dir " + fileTemp + " because of a file with the same name ");
					return false;
				}
			}
			
		}
		catch (Exception e) 
        {
			if (m_bDebug) System.err.println ("failure mkdirs " + strFileName);
			
			return false;
		}
		
		return true;
		
	}
	
	/**
	 * Write the lock token to a file
	 */
	void putLockToken () 
    {
		String strFileName = new String (m_strWebdavPath + m_strResource + 
                                    s_strStateTokenPostfix);
		
		try 
        {
			FileOutputStream outFile = new FileOutputStream (strFileName);
			DataOutputStream outData = new DataOutputStream (outFile);
			
			if (outData != null) 
            {
				outData.writeBytes (m_httpResponse.getHeader("Lock-Token"));
				outData.writeBytes ("\n");
				outFile.close();
			}
			else 
            {
				m_bErrorFlag = true;
			}
		}
		catch (IOException e) 
        {
			if (m_bDebug) System.err.println ("IOException in writing " + strFileName);
			m_bErrorFlag = true;
		}
	}
	
	/**
	 * Send the request to the webdav server
	 */
	void sendRequest () 
    {
        if (m_bErrorFlag)
        {
            return;
        }

		fillHeaders ();

//        for (int i = 0; i < m_aryHeaderPairs.length; ++i)
//        {
//            NVPair pair = (NVPair) m_aryHeaderPairs[i];
//            System.out.println("HEADER: " + pair.getName() + "=" + 
//                pair.getValue());
//        }

		try 
        {
			if (m_strMethod.equals (WebdavMethod.strGetMethod)) 
            {
				m_httpResponse = m_httpConnection.Get (m_strResource);
			}
			else if (m_strMethod.equals (WebdavMethod.strPutMethod)) 
            {
				getData (m_strResource);
				m_httpResponse = m_httpConnection.Put (m_strResource, m_buf, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strCopyMethod)) 
            {
				m_httpResponse = m_httpConnection.Copy (m_strResource, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strMoveMethod)) 
            {
				m_httpResponse = m_httpConnection.Move (m_strResource, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strPostMethod)) 
            {
				m_httpResponse = m_httpConnection.Post (m_strResource, m_aryHeaderPairs );
			}
			else if (m_strMethod.equals (WebdavMethod.strDeleteMethod)) 
            {
				m_httpResponse = m_httpConnection.Delete (m_strResource, m_aryHeaderPairs);
			}				
			else if (m_strMethod.equals (WebdavMethod.strPropFindMethod)) 
            {
				m_httpResponse = m_httpConnection.PropFind (m_strResource, null, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strPropPatchMethod)) 
            {
				String xmlFile;
				int i=0,j=0;
				// reconstruct the header because XML_file
				// should not be in the request header
				NVPair temp_headers[]  = new NVPair[m_aryHeaderPairs.length-1];
			    while (i<m_aryHeaderPairs.length) 
                {
					xmlFile = new String (m_aryHeaderPairs[i].getName ()); 
					if (xmlFile.equals ("XML_file")) 
                    {
						xmlFile = new String (m_aryHeaderPairs[i].getValue ());
						if (xmlFile.indexOf("/") != 0) 
                        {
							xmlFile = new String ("/"+xmlFile); 
						}
						getData (xmlFile);
					}
					else 
                    {
						// XML_file is not supposed to be in request header
						temp_headers[j] = new NVPair(m_aryHeaderPairs[i].getName(),
							m_aryHeaderPairs[i].getValue());
						++j;
					}
					++i;
				}
				m_aryHeaderPairs = temp_headers;
			    m_httpResponse = m_httpConnection.PropPatch (m_strResource, m_buf, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strLockMethod)) 
            {
				m_httpResponse = m_httpConnection.Lock (m_strResource, null, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strUnlockMethod)) 
            {
				m_httpResponse = m_httpConnection.Unlock (m_strResource, null, m_aryHeaderPairs);
			}
			else if (m_strMethod.equals (WebdavMethod.strMkcolMethod))
			{
				m_httpResponse = m_httpConnection.Mkcol (m_strResource, m_aryHeaderPairs);
			}
		}
		catch (IOException e) 
        {
			System.err.println ("IOException occurs in sending request: ");
			e.printStackTrace();
	  
			m_bErrorFlag = true;
		}
		catch (AuthTypeNotImplementedException e) 
        {
			System.err.println ("AuthTypeNotImplementedException occurs in sending request: ");
			e.printStackTrace();
			m_bErrorFlag = true;
		}

        System.out.println();
	}

    // process response 
    /**
     * Process the response returned from the webdav server
     */

    String procRsp () 
    {
	    String requestBody = new String(getRequestBody ());
	    StringBuffer buff = new StringBuffer ("");
	    
	    if (m_bErrorFlag) 
        {
		    return "Webdav client internal error, request not sent! \n" + requestBody +"See console for more info.\n";
	    }
	    
	    try 
        {
            if (m_httpResponse.getStatusCode() < 300) 
            {
    	        if (m_strMethod.equals(WebdavMethod.strLockMethod)) 
                {
			        putLockToken ();
		        }

                if (m_strMethod.equals(WebdavMethod.strGetMethod))
	            {
		            putData (m_strResource);
                }
            }
	    }
	    catch (IOException e) 
        {
		    if (m_bDebug) System.err.println ("Error getting the status code from the http Rsp ");
		    if (m_bDebug) System.err.println ("data from get method is not processed correctly ");
	    }
	    
	    buff.append (getResponseBody());
	    buff.append (getRequestBody());
	    
	    return buff.toString();
	    
    }

    /**
    * Gather the request information
    * @return the request information in a string format
    */
    String getRequestBody () 
    {
	    StringBuffer buf = new StringBuffer ("Request : \n" + m_strMethod + " " + m_strResource + " HTTP/1.1" + "\n"+ "Host : " + m_strHostName +"\n");
	    
	    for (int i=0; i < m_aryHeaderPairs.length; ++i) 
        {
		    buf.append (m_aryHeaderPairs[i].getName() + " : " + m_aryHeaderPairs[i].getValue() + "\n");
	    }
	    
	    if (!m_strMethod.equals(WebdavMethod.strPutMethod) && m_buf != null) 
        {
		    buf.append ("\n");
		    ByteArrayOutputStream streamBytes = new ByteArrayOutputStream();
		    byte[] aryBytes = m_buf;
		    streamBytes.write(aryBytes, 0, aryBytes.length);
		    buf.append (streamBytes.toString());
		    
	    }
	    buf.append ("\n"+"\n");
	    
	    return buf.toString();
	    
    }


    /**
    * Gather the response information
    * @return the response information in a string format
    */
    String getResponseBody () 
    {
	    StringBuffer buff = new StringBuffer ("");	  
	    
	    try 
        {
		    buff.append ("Response : "+"\n");
		    buff.append ("HTTP/1.1 " + m_httpResponse.getStatusCode()+ " " + m_httpResponse.getReasonLine());
		    buff.append ("\n"+"\n");
		    
		    if (m_httpResponse.getStatusCode() < 300) 
            {
			    if (m_strMethod.equals (WebdavMethod.strPropFindMethod) ||
				    m_strMethod.equals (WebdavMethod.strPropPatchMethod)) 
                {
				    buff.append ("Content-Type" + " : " + m_httpResponse.getHeader("Content-Type"));
				    buff.append ("\n");
				    buff.append ("Content-Length" + " : " + m_httpResponse.getHeader("Content-Length"));
				    buff.append ("\n");
				    
				    // append the data
				    
				    ByteArrayOutputStream streamBytes = new ByteArrayOutputStream();
				    byte[] aryBytes = m_httpResponse.getData();
				    streamBytes.write(aryBytes, 0, aryBytes.length);
				    buff.append (streamBytes.toString());
				    
			    }
			    else if (m_strMethod.equals (WebdavMethod.strLockMethod)) 
                {
				    buff.append ("Lock-Token" + " : " + m_httpResponse.getHeader ("Lock-Token"));
				    buff.append ("\n");
				    buff.append ("TimeOut" + " : " + m_httpResponse.getHeader ("TimeOut"));
				    buff.append ("\n");
			    }
		    }
		    
		    buff.append ("\n\n");
	    }

	    catch (IOException e) 
        {
		    System.out.println ("Exception occurs while intepreting response");
		    buff = new StringBuffer ("Internal Client Error\n" + "See console for more information ");
		    return buff.toString ();
	    }

	    if (m_bErrorFlag) 
		    buff.append ("Error occur during processing response, see console for more information \n");
	    return buff.toString();
    }

	/**
	 * Default headers for the methods
	 * @param strMethod : name of the method
	 * @param vecHeaderNames : to be filled with header names
	 * @param vecHeaderValues : to be filled with header values
	 */
	static void getHeaders (String strMethod, Vector vecHeaderNames, Vector vecHeaderValues) 
    {
		if (strMethod.equals (WebdavMethod.strMoveMethod)) 
        {
			vecHeaderNames.addElement (new String ("Destination"));
			vecHeaderValues.addElement (new String (""));
			
			vecHeaderNames.addElement (new String ("Overwrite"));
			vecHeaderValues.addElement (new String ("F")); // _F_ , if not "false", no send
			
			vecHeaderNames.addElement (new String ("Depth"));
			vecHeaderValues.addElement (new String ("0"));	 // if nothing, don't send
			
			return;
		}
		else if (strMethod.equals (WebdavMethod.strCopyMethod)) 
        {
			vecHeaderNames.addElement (new String ("Destination"));
			vecHeaderValues.addElement (new String (""));
			
			vecHeaderNames.addElement (new String ("Overwrite"));
			vecHeaderValues.addElement (new String ("F")); // _F_ ,if not "false", no send
			
			vecHeaderNames.addElement (new String ("Depth"));
			vecHeaderValues.addElement (new String ("0"));	 // if nothing, don't send
			
			return;
		}
		else if (strMethod.equals (WebdavMethod.strPropFindMethod)) 
        {
			vecHeaderNames.addElement (new String ("Propfind"));
			vecHeaderValues.addElement (new String ("allprop"));
			
			vecHeaderNames.addElement (new String ("Depth"));
			vecHeaderValues.addElement (new String ("0"));	 // if nothing, don't send
			
			return;
		}
		else if (strMethod.equals (WebdavMethod.strPropPatchMethod)) 
        {
			vecHeaderNames.addElement (new String ("XML_file"));
			vecHeaderValues.addElement (new String ("")); // the file name contain xml stuff
			
			vecHeaderNames.addElement (new String ("Content-Type"));
			vecHeaderValues.addElement (new String ("text/xml")); 
			
			return;
		}
		else if (strMethod.equals (WebdavMethod.strLockMethod)) {
			// the following combined to the header Lock-Info
			vecHeaderNames.addElement (new String ("LockType"));
			vecHeaderValues.addElement (new String ("Write"));
			
			vecHeaderNames.addElement (new String ("LockScope"));
			vecHeaderValues.addElement (new String ("Exclusive"));
			
			return;
		}

	}
}
