/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import java.security.MessageDigest;

/**
 */

public class DigestHelper
{
	// DATA

	/**
	 * Helper function that handles converting a string into
	 * a byte array, obtaining the digest for the byte array,
	 * and then converting the byte array back into a string.
	 *
	 * @param	messageDigest	[IN] A MessageDigest object
	 *                          used to calculate the digest.
	 * @param	strMessage		[IN] Arbitrary string message
	 *                          for which the digest will be
	 *                          calculated.
	 * @return	Returns the digest for the message as a
	 *			sequence of 32 hexidecimal characters.
	 */
	public static String Hash(	MessageDigest messageDigest,
								String strData)
	{
		byte[] bytesIn = new byte[strData.length()];
		strData.getBytes(0, strData.length(), bytesIn, 0);

		messageDigest.update(bytesIn);
		byte[] bytesDigest = messageDigest.digest();

		return ByteToHexString(bytesDigest);
	}

	/**
	 * Helper function that handles converting a string into
	 * a byte array, obtaining the digest for the byte array,
	 * and then converting the byte array back into a string.
	 *
	 * @param	messageDigest	[IN] A MessageDigest object
	 *                          used to calculate the digest.
	 * @param	strMessage		[IN] Arbitrary string message
	 *                          for which the digest will be
	 *                          calculated.
	 * @return	Returns the digest for the message as a
	 *			sequence of 32 hexidecimal characters.
	 */
	public static String Hash(	MessageDigest messageDigest,
								String strData,
								String strSecret)
	{
		return Hash(messageDigest, strSecret + ":" + strData);
	}

	/**
	 * Helper function used by StringToDigest() to convert a
	 * byte array into a hexidecimal encoded string.  Each
	 * byte will be encoded as two hexidecimal characters
	 * representing the first and second 4-bits of the byte.
	 * For example, the byte (00001111) would be encoded as
	 * "0F".
	 *
	 * @param	bytes	[IN] Byte array to be encoded.
	 * @return	Returns the hexidecimal encoding of the byte
	 *          array with length equal to twice the byte
	 *          array length.
	 */
	public static String ByteToHexString(byte[] bytes)
	{
		StringBuffer strbuf = new StringBuffer();

		for (int i=0; i < bytes.length; ++i)
		{
			int nSecondHexDigit  = (bytes[i] >> 0) & 0x0F;
			int nFirstHexDigit = (bytes[i] >> 4) & 0x0F;

			strbuf.append(Character.forDigit(nFirstHexDigit, 16));
			strbuf.append(Character.forDigit(nSecondHexDigit, 16));
		}

		return new String(strbuf);
	}

	/**
	 * Test driver for this class.
	 */
	public static final void main(String[] args)
	{
	}
};