/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.StringTokenizer;

/**
 *
 * @author		Marc Eaddy
 * @version		1.0, 25 Apr 1998
 */

public class HeaderHelper
{
	/**
	 * Helper function for displaying a name=value pair.
	 *
	 * @param	strName		[IN] Attribute name.
	 * @param	strValue	[IN] Attribute value.  Can be
	 *						empty or null.
	 * @return	"strName="strValue"" if strValue is non-empty,
	 *			otherwise returns "",
	 */
	public static String paramToString(	String strName, 
										String strValue)
	{
		if (strValue != null && !strValue.equals(""))
		{
			return new String( strName.toLowerCase() + "=\"" + strValue + "\"");
		}
		else
		{
			return new String("");
		}
	}

	public static void parseHeaderProperties(	String strRawHeader,
												Hashtable mapProperties)
	{
		StringTokenizer strtok = new StringTokenizer(strRawHeader);

		while (strtok.hasMoreElements())
		{
			String str = new String(strtok.nextToken(", \t"));

			if (!str.equals(""))
			{
				parseNameValuePair(str, mapProperties);
			}
		}
	}

	/**
	 * Helper function that takes a vector of name=value strings
	 * and initializes the appropriate member variables with them.
	 */
	public static void parseNameValuePair(	String strNameValuePair, 
											Hashtable mapProperties)
	{
		// Start with NameValuePair string (Ex: "joe="jack"")
		StringTokenizer strtok = 
				new StringTokenizer(	strNameValuePair,
										"=\" \t"); 

		// Get the name (Ex: "joe")
		String strName = strtok.nextToken("= \t").trim();

		if (!strtok.hasMoreElements())
		{
			return;
		}

		// Get the value (Ex: ""jack"");
		String strValue = strtok.nextToken();

		// Trim off the double quotes (Ex: "jack")
		if (strValue.length() >= 2 &&
			strValue.charAt(0) == '"' &&
			strValue.charAt(strValue.length() - 1) == '"')
		{
			strValue = strValue.substring(1, strValue.length() - 1);
		}
		else
		{
			return;
		}

		strName = strName.toLowerCase();

		String strMappedValue = (String) mapProperties.get(strName);

		if (strMappedValue == null)
		{
			mapProperties.put(strName, strValue);
		}
		else
		{
			// Blow up?
			return;
		}
	}

	public static String mapToString(Hashtable mapProperties)
	{
		String strOutput = new String();

		boolean bFirst = true;
		Enumeration enumMapKeys = mapProperties.keys();
		while (enumMapKeys.hasMoreElements())
		{
			String strKey   = (String) enumMapKeys.nextElement();
			String strValue = (String) mapProperties.get(strKey);
			
			if (strKey.equals("") ||
				strValue == null || 
				strValue.equals(""))
			{
				continue;
			}

			if (!bFirst)
			{
				strOutput += ", ";
			}

			strOutput += paramToString(strKey, strValue);

			bFirst = false;
		}

		return strOutput;
	}

	/**
	 * Test driver for this class.
	 */
	public static final void main(String[] args)
	{
	}
};