/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.Vector;

/**
 *
 * @author		Marc Eaddy
 * @version		1.0, 25 Apr 1998
 */

public class HeaderWithParameters
{
	// DATA

    private Vector vecProperties = new Vector();
    protected String strHeaderName = null;

	/**
	 * Creates a HeaderWithParameters object.
	 */
	public HeaderWithParameters(String strHeaderName)
	{ 
		this.strHeaderName = new String(strHeaderName);
	}

	/**
	 * Parses the "Authentication" HTTP header.  It is
	 * assumed that the header does not contain the header
	 * name.  This is the normal behaviour when retrieving
	 * a header from HttpRequest or HttpResponse using
	 * getHeader().
	 *
	 * <P> This is the CTOR used by the server to parse a
	 * Authentication header received from a client.
	 *
	 * @see HttpRequest
	 * @see HttpResponse
	 */
	public HeaderWithParameters(String strHeaderName,
								String strHeaderWithParameters)
	{
		this(strHeaderName);

		parseHeaderProperties(strHeaderWithParameters);
	}

	/**
	 * COPY CTOR
	 */
	public HeaderWithParameters(HeaderWithParameters that)
	{ 
		if (that != null && this != that)
		{
			this.strHeaderName = new String(that.strHeaderName);

			Enumeration enumElems = that.vecProperties.elements();
			while (enumElems.hasMoreElements())
			{
				HeaderParam param = (HeaderParam) enumElems.nextElement();
				setParam( param.getName(), param.getValue() );
			}
		}
	}

	/**
	 *
	 */
	public HeaderWithParameters(String strHeaderName,
                                HeaderWithParameters that)
	{ 
        this(that);

        this.strHeaderName = new String(strHeaderName);
    }

	/**
	 * Allows you to set a property name and value.
	 */
    protected void setParam(String strName, String strValue)
    {
        HeaderParam param = new HeaderParam(strName, strValue);

        int nIndex = vecProperties.indexOf(param);

        if (nIndex == -1)
        {
            vecProperties.addElement(param);
        }
        else
        {
            HeaderParam paramFound = 
                (HeaderParam) vecProperties.elementAt(nIndex);
            paramFound.setValue(strValue);
        }
    }

	/**
	 * @return	Returns the official header name for this
	 *			header (i.e., "Authentication").
	 */
	public String getHeaderName()
	{
		return strHeaderName;
	}

	/**
	 * @return	The header property value or null if a value
	 *			is not defined for this property name.
	 */
	public String getParam(String strName)
	{
        HeaderParam param = new HeaderParam(strName, "");

        int nIndex = vecProperties.indexOf(param);

        if (nIndex == -1)
        {
            return null;
        }
        else
        {
            HeaderParam paramFound = 
                (HeaderParam) vecProperties.elementAt(nIndex);
            
            return paramFound.getValue();
        }
	}

    /**
     * @return  The header params for this header.  Each
     *          element is of type HeaderParam.
     */
    public Enumeration getHeaderParams()
    {
        return vecProperties.elements();
    }

    /**
     */
    protected String paramsToString()
    {
        String strOutput = new String();

        for (int i = 0; i < vecProperties.size(); ++i)
        {
            HeaderParam param = (HeaderParam) vecProperties.elementAt(i);

            if (i > 0)
            {
                strOutput += ", ";
            }

            if (param.getValue() != null)
            {
                strOutput += param.getName() + 
                    "=\"" + param.getValue() + "\"";
            }
        }
        
        return strOutput;
    }

	/**
	 * Converts the AuthenticationHeader into a string suitable
	 * for transmission as an HTTP header.
	 */
	public String toString()
	{
		return new String(paramsToString());
	}

	public boolean equals(Object objRhs)
	{
		HeaderWithParameters rhs = (HeaderWithParameters) objRhs;
        HeaderWithParameters lhs = this;

        if (this.vecProperties.size() != rhs.vecProperties.size())
        {
            // LHS != RHS
            return false;
        }

		Enumeration enumRhs = rhs.vecProperties.elements();
        Enumeration enumLhs = lhs.vecProperties.elements();
		
        while ( enumRhs.hasMoreElements() &&
                enumLhs.hasMoreElements() )
		{
            HeaderParam paramRhs = (HeaderParam) enumRhs.nextElement();
            HeaderParam paramLhs = (HeaderParam) enumLhs.nextElement();

            String strRhs = paramRhs.getValue();
            String strLhs = paramLhs.getValue();

            if (!paramLhs.getName().equals(paramRhs.getName()) ||
                !EqualOrBothNull(strLhs, strRhs))
            {
                // LHS != RHS
                return false;
            }
		}

        // LHS == RHS
        return true;
	}

    private boolean EqualOrBothNull(String strLhs, String strRhs)
    {
        if (strLhs == null || strRhs == null)
        {
            return strLhs == strRhs;
        }
        else
        {
            return strLhs.equals(strRhs);
        }
    }

	protected void parseHeaderProperties(String strRawHeader)
	{
        StringTokenizer strtok = new StringTokenizer(strRawHeader, ",");

        while (strtok.hasMoreElements())
        {
            String strToken = new String(strtok.nextToken());

            int nEqualSignPos = strToken.indexOf('=');

            if (nEqualSignPos == -1)
            {
                continue;
            }

            // Find the beginning of the parameter's name

            // Walk backward until you find the first whitespace character
            int nBegPos;
            for (   nBegPos = nEqualSignPos; 
                    nBegPos > 0 &&
                    !Character.isSpace(strToken.charAt(nBegPos-1));
                    --nBegPos) ;

            // Find the ending of the parameter's value
            int nEndPos = strToken.indexOf('\"', nEqualSignPos + 2);

            // Could not find it
            if (nBegPos < 0 || 
                nEndPos == -1 || 
                strToken.charAt(nEqualSignPos + 1) != '\"')
            {
                continue;
            }
            
            String strName = 
                new String(strToken.substring(nBegPos, nEqualSignPos));
            String strValue = 
                new String(strToken.substring(nEqualSignPos + 2, nEndPos));

            setParam(strName, strValue);
        }
	}

	/**
	 * Test driver for this class.
	 */
	public static void main(String[] args)
	{
        HeaderWithParameters hdr = new HeaderWithParameters("Marc");

        System.out.println(hdr);
	}
};

class HeaderParam
{
    // CREATORS

    public HeaderParam(String strName, String strValue)
    {
        this.strName = new String(strName.toLowerCase());

        if (strValue != null)
        {
            this.strValue = new String(strValue);
        }
    }

    // ACCESSORS

    public String getValue()
    {
        return strValue;
    }

    public String getName()
    {
        return strName;
    }

    // MANIPULATORS

    public void setValue(String strValue)
    {
        this.strValue = new String(strValue);
    }

    public boolean equals(Object that)
    {
        if (this == that) 
        {
            return true;
        }

        else if (that == null || this.getClass() != that.getClass()) 
        {
            return false;
        }

        else
        {
		    HeaderParam hdrThat = (HeaderParam) that;

            return this.strName.equals(hdrThat.strName);
        }
    }

    // DATA
    private String strName = null;
    private String strValue = null;
};
