/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import webdav.common.UriScope;

/**
 * Associates a subsystem type to a scope of uri's.  This allows us
 * to have multiple subsystems with the same type but having different
 * uri scopes.  For example, we could have two Lock subsystems, one that
 * uses NT System Security to perform file locking for the
 * "http://www.webdav.org/ntfiles/.*" uri scope and one that uses Unix
 * file locking for the "http://www.webdav.org/unix_files/.*" uri scope.
 *
 * <P>The SubsystemRegistry associates actual SubsystemInterfaces with
 * their SubsystemScope.
 *
 * @see			SubsystemRegistry
 * @see			SubsystemInterface
 * @see			UriScope
 * @author		Marc Eaddy
 * @version		1.0, 3 Nov 1997
 */
public class SubsystemScope
{
	private String		strType;
	private UriScope	scopeUri;

	public SubsystemScope(String strType, UriScope scopeUri)
	{
		this.strType	= new String(strType);
		this.scopeUri	= (UriScope) scopeUri.clone();
	}

	public String getType()
	{
		return strType;
	}

	public UriScope getScope()
	{
		return scopeUri;
	}

	public String toString()
	{
		return strType + " Subsystem, Scope = " + scopeUri;
	}

	public int hashCode()
	{
		// Not correct!  This could possible return the same value for
		// two different scopes.  Fix this later.
		return strType.hashCode() + scopeUri.hashCode();
	}

	public boolean equals(Object objRhs)
	{
		SubsystemScope rhs = (SubsystemScope) objRhs;

		return (strType.equals(rhs.strType) && scopeUri.equals(rhs.scopeUri));
	}
};
