/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

/**
 * Allows multiple uri's to be referred to by a regular expression.<P>
 *
 * Here are some valid UriScope's:
 *		
 * <UL>
 *		<LI> http://www.tvguide.com/.*
 *		<LI> http://www.tvguide.com/tv/covers/images/*.gif
 *		<LI> http://www.tvguide.com/tv/listings/ny/06.sml
 * </UL>
 * 
 * <P>Note, the last UriScope referes to exactly one uri so isOne() would
 * return true.
 *
 * @see			Uri
 * @author		Marc Eaddy
 * @version		1.0, 3 Nov 1997
 *
 */
public class UriScope
	implements Cloneable
{
	private String strScope;

	/**
	 * Checks to see if the scope is actually one uri.  This would
	 * be true for "http://www.tvguide.com/index.html" but not for
	 * "http://www.tvguide.com/.*".
	 *
	 * @return		true if the scope resolves to exactly one uri;
	 *              false otherwise
	 */
	public boolean isOne()
	{
		return true;
	}

	public UriScope(String strScope)
	{
		this.strScope = new String(strScope);
	}

	public Object clone()
	{
		try
		{
			return super.clone();
		}
		catch (CloneNotSupportedException ex)
		{
			// Cannot happen -- we support clone()
			throw new InternalError(ex.toString());
		}
	}

	public boolean equals(Object objRhs)
	{
		UriScope rhs = (UriScope) objRhs;

		return ( strScope.equals(rhs.strScope) );
	}

	public String toString()
	{
		return strScope;
	}

	public int hashCode()
	{
		return strScope.hashCode();
	}
};