/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.common;

import com.ms.xml.util.Name;
import com.ms.xml.om.Element;
import com.ms.xml.om.ElementImpl;
import java.util.Enumeration;

/**
 * Define the Generic DAV XML Elements from section 12 of the WebDAV spec
 * <draft-ietf-webdav-protocol-05>
 *
 * @author		Jonathan Shapiro
 * @version		1.0, 23 Nov 1997
 */
public class WebdavXML
{
	public static final ElementImpl elemNewline =
			new ElementImpl(null, Element.WHITESPACE);

	public static final ElementImpl elemTab =
			new ElementImpl(null, Element.WHITESPACE);

	static
	{
		elemNewline.setText("\n");
		elemTab.setText("\t");
	}

	public WebdavXML()
	{
	}

	//8.2.1
	public static final Name ELEM_PROPERTYUPDATE = Name.create("propertyupdate");

	//8.2.2
	public static final Name ELEM_SET = Name.create("set");

	//8.2.3
	public static final Name ELEM_REMOVE = Name.create("remove");

	//11.1
	public static final Name ELEM_MULTISTATUS = Name.create("multistatus");

	//11.2
	public static final Name ELEM_RESPONSE = Name.create("response");

	//11.3
	public static final Name ELEM_STATUS = Name.create("status");

	//11.4
	public static final Name ELEM_RESPONSEDESCRIPTION = Name.create("responsedescription");


	//12.1
	public static final Name ELEM_HREF = Name.create("href");

	//12.2
	public static final Name ELEM_LINK = Name.create("link");
	//12.2.1
	public static final Name ELEM_SRC = Name.create("src");
	//12.2.2
	public static final Name ELEM_DST = Name.create("dst");

	//12.3
	public static final Name ELEM_PROP = Name.create("prop");

	//13.14
	public static final Name ELEM_RESOURCE_TYPE = Name.create("resourcetype");

	//13.14.1
	public static final Name ELEM_COLLECTION_RESOURCE_TYPE = Name.create("collection");

	public static Name getNonNullTagName(Element elem) {
		Name name = null;
		Enumeration enum = elem.getElements();
		while (enum.hasMoreElements()) {
			Element elemTmp = (Element)enum.nextElement();
			name = elemTmp.getTagName();
			if (name != null) break;
		}
		return name;
	}
}