/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.Uri;
import webdav.common.WebDavRetCode;
import webdav.common.SubsystemRegistry;
import webdav.authentication.AuthenticationInterface;
import webdav.authentication.WWWAuthenticateHeader;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.Enumeration;

import javax.servlet.http.HttpServletResponse;

/**
 *
 * @see			webdav.executive.Command
 * @see         webdav.authentication.AuthenticationInterface
 * @author		Marc Eaddy
 * @version		1.0, 2 May 1998
 */
public class AuthenticateCmd
	extends Command
{
	private AuthenticationInterface ifaceAuthentication = null;
        // Should be static

    private String  strAuthorizationHeader      = null;
    private String  strRemoteIPAddress          = null;
    private String  strRealm                    = null;
    private String  strMethod                   = null;
	private Uri     uriRequest                  = null;
    private Vector  vecWWWAuthenticateHeaders   = new Vector();

	/**
	 * Create and initialize the GetResourceCmd command.
	 *
	 * @param uri		[IN] Location of the resource file
	 * @param streamOut [OUT] Output stream which will
	 *					contain the contents of the resource file
	 */
    public AuthenticateCmd( String strAuthorizationHeader, 
                            String strRemoteIPAddress,
                            String strRealm,
                            String strMethod,
                            Uri uriRequest)
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
        if (strAuthorizationHeader != null)
        {
            this.strAuthorizationHeader = new String(strAuthorizationHeader);
        }
        else
        {
            this.strAuthorizationHeader = null;
        }

        this.strRemoteIPAddress = new String(strRemoteIPAddress);
        this.strRealm           = new String(strRealm);
        this.strMethod          = new String(strMethod);
		this.uriRequest         = new Uri(uriRequest);

		//*** Connect to the Namespace Manager ***

		ifaceAuthentication = (AuthenticationInterface) 
			SubsystemRegistry.lookup( "Authentication", uriRequest );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		return ifaceAuthentication.authenticate(strAuthorizationHeader,
                                                strRemoteIPAddress,
                                                strRealm,
                                                strMethod,
                                                uriRequest.toString(),
                                                vecWWWAuthenticateHeaders);
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		return WebDavRetCode.WEBDAV_SUCCESS;
	}

    public void sendWWWAuthenticateHeaders(HttpServletResponse res)
    {
        Enumeration enumHdrs = vecWWWAuthenticateHeaders.elements();
        while (enumHdrs.hasMoreElements())
        {
            WWWAuthenticateHeader hdrWWWAuth = 
                (WWWAuthenticateHeader) enumHdrs.nextElement();

            res.setHeader(  hdrWWWAuth.getHeaderName(),
                                hdrWWWAuth.toString());
            System.out.println("Sending WWW-Authenticate header: " +
                                hdrWWWAuth);
        }
    }
};