/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import com.ms.xml.om.*;
import com.ms.xml.util.Name;
import webdav.common.*;
import webdav.propman.PropertyManager;	// Only deals with the iface

/**
 * The CopyPropCmd command copies the properties of the source
 * resource to the destination resource.
 * <P>
 * Undo of this command reverts the destination properties to
 * their original value.
 *
 * @see			webdav.executive.Command
 * @see         webdav.propman.PropertyManager
 * @see         webdav.executive.CopyMethod
 * @author		Jonathan Shapiro
 * @version		1.0, 23 Nov 1997
 */
public class CopyPropCmd
	extends PropCmd
{

	/**
	 * Create and initialize the command.
	 *
	 */
	public CopyPropCmd(Uri uriSrc, Uri uriDest, boolean bOverwrite, Depth depth)
	{
		_uriSrc = uriSrc;
		_uriDest = uriDest;
		_depth = depth;
		_bOverwrite = bOverwrite;

		_newDestProp = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);
		_origDestProp = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);

		_ifacePropMan = (PropertyManager)SubsystemRegistry.lookup("Property", 
																	_uriSrc);
	}

	/**
	 * @see Command
	 */
	protected int DoImpl()
	{
		// Get current destination properties to support Undo()
		int nGetDestStatus = _ifacePropMan.getAll(_uriDest, _origDestProp);

		_undoStatus = nGetDestStatus;

		if (nGetDestStatus < WebDavRetCode.WEBDAV_SUCCESS)
		{
			return nGetDestStatus;
		}

		// Copy the source properties to the destination
		int nCopyStatus = _ifacePropMan.copyAll( _uriSrc, _uriDest, _newDestProp);

		_propManStatus = nCopyStatus;

		return _propManStatus;
	}

	/**
	 * @see Command
	 */
	protected int UnDoImpl()
	{ 
		if (_undoStatus < WebDavRetCode.WEBDAV_SUCCESS)
		{
			return _undoStatus;
		}
		else
		{
			return _ifacePropMan.setProp(_uriDest, _origDestProp, _newDestProp);
		}
	}

	/**
	 *@see PropCmd
	 */
	public String getHref() {
		return _uriSrc.toString();
	}

	/**
	 *@see PropCmd
	 */
	public Element getOutput() {
		return _newDestProp;
	}

	/**
	 *@see PropCmd
	 */
	public int getInternalStatusCode() {
		return _propManStatus;
	}

	private Uri _uriSrc = null;
	private Uri _uriDest = null;
	private Depth _depth = null;
	private boolean _bOverwrite = true;
	private PropertyManager _ifacePropMan = null;
	private Element _newDestProp = null;
	private Element _origDestProp = null;
	private int _propManStatus = PropertyManager.pm_success;
	private int _undoStatus = PropertyManager.pm_success;
}