/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import com.ms.xml.om.*;
import com.ms.xml.util.Name;
import webdav.common.*;
import webdav.propman.PropertyManager;	// Only deals with the iface

/**
 * The DeleteAllPropCmd command deletes all the properties defined
 * on the resource. Undo of this command resets the properties to 
 * their original value.
 *
 * @see			Command
 * @see         PropertyManager
 * @author		Jonathan Shapiro, Marc Eaddy
 * @version		1.0, 23 Nov 1997
 */
public class DeleteAllPropCmd
	extends PropCmd
{

	/**
	 * Create and initialize the command.
	 *
	 * @param uri		[IN] Location of the resource file
	 * @param streamOut [OUT] Output stream which will
	 *					contain the contents of the resource file
	 */
	public DeleteAllPropCmd(Uri uri)
	{
		_uri = uri;
		_outProp = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);
		_inPropUndo = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);
		_outPropUndo = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);
		_restorePropUndo = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);

		_ifacePropMan = (PropertyManager)SubsystemRegistry.lookup("Property", uri);
	}

	/**
	 * @see Command
	 */
	protected int DoImpl()
	{
		//get undo information
		_undoStatus = _ifacePropMan.getProp(_uri, _inPropUndo, _restorePropUndo);

		//execute the command
		_propManStatus = _ifacePropMan.deleteAll(_uri);

		return _propManStatus;
	}

	/**
	 * @see Command
	 */
	protected int UnDoImpl()
	{ 
		if (_undoStatus == PropertyManager.pm_success)
			return _ifacePropMan.setProp(_uri, _restorePropUndo, _outPropUndo);
		else
			return PropertyManager.pm_success;

	}

	/**
	 *@see PropCmd
	 */
	public String getHref() {
		return _uri.toString();
	}

	/**
	 *@see PropCmd
	 */
	public Element getOutput() {
		return _outProp;
	}

	/**
	 *@see PropCmd
	 */
	public int getInternalStatusCode() {
		return _propManStatus;
	}

	private Uri _uri = null;
	private PropertyManager _ifacePropMan = null;
	private Element _inProp = null;
	private Element _outProp = null;
	private Element _inPropUndo = null;
	private Element _outPropUndo = null;
	private Element _restorePropUndo = null;
	private int _propManStatus = PropertyManager.pm_success;
	private int _undoStatus = PropertyManager.pm_success;
}