/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import com.ms.xml.om.*;
import com.ms.xml.util.Name;
import webdav.common.*;
import webdav.propman.PropertyManager;	// Only deals with the iface

/**
 * The GetPropCmd command retrieves the value of a specified property
 * on a resource.
 *
 * @see			Command
 * @see         PropertyManager
 * @author		Jonathan Shapiro
 * @version		1.0, 23 Nov 1997
 */
public class GetPropCmd
	extends PropCmd
{

	/**
	 * Create and initialize the GetResource command.
	 *
	 * @param uri		[IN] Location of the resource file
	 * @param streamOut [OUT] Output stream which will
	 *					contain the contents of the resource file
	 */
	public GetPropCmd(Uri uri, Name prop)
	{
		_uri = uri;
		_prop = prop;
		_inProp = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);
		_outProp = new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);

		/*Note: Changing the PropertyManager.getProp function to take a
		  Name instead of a <prop> element would make this queue take much less
		  space. Could be important for deep tree operations !!
		*/
		_inProp.addChild(new ElementImpl(prop, Element.ELEMENT), null);

		_ifacePropMan = (PropertyManager)SubsystemRegistry.lookup("Property", uri);
	}

	/**
	 * @see Command
	 */
	protected int DoImpl()
	{
		_propManStatus = _ifacePropMan.getProp(_uri, _inProp, _outProp);

		return _propManStatus;
	}

	/**
	 * @see Command
	 */
	protected int UnDoImpl()
	{ 
		return WebDavRetCode.WEBDAV_SUCCESS;
	}

	/**
	 *@see PropCmd
	 */
	public String getHref() {
		return _uri.toString();
	}

	/**
	 *@see PropCmd
	 */
	public Element getOutput() {
		return _outProp;
	}

	/**
	 *@see PropCmd
	 */
	public int getInternalStatusCode() {
		return _propManStatus;
	}

	private Uri _uri = null;
	private Name _prop = null;
	private PropertyManager _ifacePropMan = null;
	private Element _inProp = null;
	private Element _outProp = null;
	private int _propManStatus = PropertyManager.pm_success;
}