/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;	// Only deals with the iface
import java.util.NoSuchElementException;
import java.io.OutputStream;

/**
 * The GetResourceCmd command retrieves the file contents
 * of a resource Uri.
 *
 * @see			Command
 * @see         NamespaceInterface
 * @author		Marc Eaddy
 * @version		1.0, 8 Nov 1997
 */
public class GetResourceCmd
	extends Command
{
	private Uri uri;
	private String strResource = new String();
	private NamespaceInterface ifaceNamespace;	// Should be static
	private OutputStream streamOut;

	/**
	 * Create and initialize the GetResourceCmd command.
	 *
	 * @param uri		[IN] Location of the resource file
	 * @param streamOut [OUT] Output stream which will
	 *					contain the contents of the resource file
	 */
	public GetResourceCmd(Uri uri, OutputStream streamOut)
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
		this.uri = new Uri(uri);
		this.streamOut = streamOut;

		//*** Connect to the Namespace Manager ***

		ifaceNamespace = (NamespaceInterface) 
			SubsystemRegistry.lookup( "Namespace", uri );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		return ifaceNamespace.getResource(uri, streamOut);
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		return WebDavRetCode.WEBDAV_SUCCESS;
	}
};