/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.WebDavResponse;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpServletRequest;

/**
 * The InvalidMethod class represents an HTTP or WebDAV method that
 * is either not understood or not implemented.
 *
 * @see			Method
 * @see         MethodFactor
 * @see         CommandQueue
 * @author		Marc Eaddy
 * @version		1.1, 14 Dec 1997
 */
public class InvalidMethod 
	extends Method
{
	public static final int METHOD_NOT_IMPLEMENTED	= 0;
	public static final int METHOD_UNKNOWN			= 1;

	private int nError;

	/**
	 * @see Method
	 */
	public InvalidMethod(int nError, HttpServletRequest req, HttpServletResponse res)
	{
		super(null, null, req, res, DO_NOT_ENFORCE_LOCKS);

		this.nError = nError;
	}

	/**
	 * Returns an HTTP status code indicating why the HTTP method
	 * failed.
	 *
	 * @return		One of the following status codes: <P>
	 *              <PRE>
	 *				SC_NOT_IMPLEMENTED			- Method is not implemented
	 *				SC_BAD_REQUEST				- Method is unknown
	 *				SC_INTERNAL_SERVER_ERROR	- Initialization error
	 *				</PRE>
	 */
	protected int getStatusCode(int nWebDavReturn)
	{
		switch (nError)
		{
		case METHOD_NOT_IMPLEMENTED: 
			return WebDavResponse.SC_NOT_IMPLEMENTED;
		case METHOD_UNKNOWN:
			return WebDavResponse.SC_BAD_REQUEST;
		default:
			return WebDavResponse.SC_INTERNAL_SERVER_ERROR;
		}
	}
};

