/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;	// Only deals with the iface
import webdav.nsman.CollectionMember;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.Enumeration;
import java.io.OutputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;	// For main()
/**
 * The ListCollectionCmd command retrieves the file contents
 * of a resource Uri.
 *
 * @see			webdav.executive.Command
 * @see         webdav.nsman.NamespaceInterface
 * @author		Marc Eaddy
 * @version		1.0, 8 Nov 1997
 */
public class ListCollectionCmd
	extends Command
{
	private Uri		uri;
	private OutputStream streamOut;

	private NamespaceInterface ifaceNamespace;	// Should be static

	/**
	 * Create and initialize the GetResourceCmd command.
	 *
	 * @param uri		[IN] Location of the resource file
	 * @param streamOut [OUT] Output stream which will
	 *					contain the contents of the resource file
	 */
	public ListCollectionCmd(Uri uri, OutputStream streamOut)
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
		this.uri = new Uri(uri);
		this.streamOut = streamOut;

		//*** Connect to the Namespace Manager ***

		ifaceNamespace = (NamespaceInterface) 
			SubsystemRegistry.lookup( "Namespace", uri );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		Vector vecMembers = new Vector();

		int nListReturn = ifaceNamespace.listCollection(uri, vecMembers);

		if (nListReturn < WebDavRetCode.WEBDAV_SUCCESS)
		{
			return nListReturn;
		}

		// Create the multi status XML element which will
		// contain a response for each directory item

		MultiStatusElement elemMultiStatus = 
			new MultiStatusElement();

		Enumeration enumMembers = vecMembers.elements();
		while (enumMembers.hasMoreElements())
		{
			CollectionMember member = 
				(CollectionMember) enumMembers.nextElement();

			elemMultiStatus.addResponse( member );
		}

		// Output the XML to the output stream

		try
		{
			elemMultiStatus.save(streamOut);
		}
		catch (IOException ex)
		{
			return WebDavRetCode.WEBDAV_FAILURE;
		}

		return nListReturn;
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		return WebDavRetCode.WEBDAV_SUCCESS;
	}

	public static final void main(String[] args)
	{
		new SubsystemRegistry();

		Uri uri = new Uri("\\");

		ListCollectionCmd cmdListCollection =
			new ListCollectionCmd(uri, System.out);

		int nRet = cmdListCollection.DoImpl();

		if (nRet < 0)
		{
			System.out.println("list failed: " + nRet);
		}
	}
};