/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;	// Only deals with the iface
import java.util.NoSuchElementException;
import java.io.ByteArrayOutputStream;
import java.io.ByteArrayInputStream;

/**
 * The MakeCollectionCmd command creates a collection
 * for the uri specified.  The undo behavior is to
 * remove the collection created.
 *
 * @see			Command
 * @see         NamespaceInterface
 * @author		Marc Eaddy
 * @version		1.0, 8 Nov 1997
 */
public class MakeCollectionCmd
	extends Command
{
	private Uri uri;
	private NamespaceInterface ifaceNamespace;	// Should be static

	/**
	 * Create and initialize the CopyResourceCmd command.
	 *
	 * @param uriSrc		[IN] Location of the source resource file
	 * @param uriDest		[IN] Location of the destination resource
	 *						file
	 * @param bOverwrite	[IN] Whether or not to overwrite the
	 *						destination resource if it already exists
	 * @param depth			[IN] 0, 1, or infinity
	 */
	public MakeCollectionCmd( Uri uri )
		throws NoSuchElementException
	{
		super();

		// Copy the input parameters
		this.uri = new Uri(uri);

		//*** Connect to the Namespace Manager ***

		// Note: This could cause a problem if the destination
		// Uri is in a different namespace manager scope.

		ifaceNamespace = (NamespaceInterface) 
			SubsystemRegistry.lookup( "Namespace", uri );
	}

	/**
	 * See Command documentation.
	 */
	protected int DoImpl()
	{
		return ifaceNamespace.makeCollection(uri);
	}

	/**
	 * See Command documentation.
	 */
	protected int UnDoImpl()
	{ 
		return ifaceNamespace.deleteResource(uri);
	}
};