/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.*;
import java.io.*;

/**
 * The MethodFactory provides a centralized place where
 * Method classes are created depending upon the particular
 * WebDavRequest.
 *
 * <P>At this level, the WebDavRequest and WebDavResponse 
 * objects are broken down or converted into input/output
 * parameters for the appropriate Method.  This keeps the
 * interfaces for the Methods simple because they only need
 * to be coupled to what they absolutely need and keeps the
 * Executive simple because it does not have to do all this
 * work.
 *
 * @see			webdav.executive.Method
 * @see         webdav.executive.Executive
 * @author		Marc Eaddy, Jonathan Shapiro, Shao Ai Rong
 * @version		2.0, 14 Dec 1997
 */
public class MethodFactory extends Object
{

	public static Method create(HttpServletRequest req, HttpServletResponse res)
	{
		String strMethod = req.getMethod();
		Uri strRequestUri = new Uri( req.getRequestURI() );
		Principal principal = new Principal(req.getRemoteUser());
		

		if (strMethod.equalsIgnoreCase("GET"))
		{
			return new GetMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("PUT"))
		{
			return new PutMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("POST"))
		{
			return new InvalidMethod(InvalidMethod.METHOD_NOT_IMPLEMENTED, req, res);
		}
		else if (strMethod.equalsIgnoreCase("MKCOL"))
		{
			return new MkcolMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("OPTIONS"))
		{
			return new InvalidMethod(InvalidMethod.METHOD_NOT_IMPLEMENTED, req, res);
		}
		else if (strMethod.equalsIgnoreCase("HEAD"))
		{
			return new InvalidMethod(InvalidMethod.METHOD_NOT_IMPLEMENTED, req, res);
		}
		else if (strMethod.equalsIgnoreCase("COPY"))
		{
			return new CopyMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("MOVE"))
		{
			return new MoveMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("DELETE"))
		{
			return new DeleteMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("PROPFIND"))
		{
			return new PropFindMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("PROPPATCH"))
		{
			return new PropPatchMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("LOCK"))
		{
			return new LockMethod(strRequestUri, principal, req, res);
		}
		else if (strMethod.equalsIgnoreCase("UNLOCK"))
		{
			return new UnlockMethod(strRequestUri, principal, req, res);
		}
		else
		{
			return new InvalidMethod(InvalidMethod.METHOD_UNKNOWN, req, res);
		}
	}
};
