/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import webdav.common.*;
import webdav.nsman.NamespaceInterface;	// Only deals with the iface
import webdav.nsman.CollectionMember;
import com.ms.xml.om.*;
import com.ms.xml.util.XMLOutputStream;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.Enumeration;
import java.io.OutputStream;
import java.io.IOException;
import java.io.ByteArrayOutputStream;	// For main()
/**
 *
 * @author		Marc Eaddy
 * @version		1.0, 20 Dec 1997
 */
public class MultiStatusElement
	extends ElementImpl
{
	/**
	 */
	public MultiStatusElement()
	{ 
		super(WebdavXML.ELEM_MULTISTATUS, Element.ELEMENT);
	}

	public void addResponse(Element elemResponse)
	{
		addChild(WebdavXML.elemNewline, null);
		addChild(WebdavXML.elemTab, null);
		addChild(elemResponse, null);
		addChild(WebdavXML.elemNewline, null);
	}

	public void addResponse(	String strHref,
								Element elemProps,
								int nHttpStatusCode)
	{
		ElementImpl elemResponse = 
			new ElementImpl(WebdavXML.ELEM_RESPONSE, Element.ELEMENT);

		Element elemHref = UriHelper.createHrefElement(strHref);

		Element elemStatus = 
			WebDavResponse.createHttpStatusElement(nHttpStatusCode);

		// Create the response element
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(elemHref,  null);
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(elemProps, null);
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(elemStatus, null);
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);

		addResponse(elemResponse);
	}

	public void addResponse(	CollectionMember member,
								int nHttpStatusCode)
	{
		ElementImpl elemProps =
			new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);

		Element elemResourceType = 
			member.createResourceTypeElement();

		// Create the prop element
		elemProps.addChild(WebdavXML.elemNewline, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(elemResourceType, null);
		elemProps.addChild(WebdavXML.elemNewline, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(WebdavXML.elemTab, null);

		addResponse( member.uri().toString(), elemProps, nHttpStatusCode);
	}

	public void addResponse( CollectionMember member )
	{
		addResponse(member, WebDavResponse.SC_OK);
	}

	public void save(OutputStream o) 
		throws IOException
	{
		// Output the XML to the output stream

		XMLOutputStream xml_out = new XMLOutputStream(o);

		super.save(xml_out);
	}
};