/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.executive;

import java.util.Hashtable;
import com.ms.xml.om.Element;
import webdav.propman.PropertyManager;
import webdav.common.WebDavResponse;

/**
 * The PropCmd inteface is implemented by the command objects that fetch information
 * from the property manager and used by the PropFindMethod class whose code to construct 
 * a multi-status response can treat all of these commands identically.
 *
 * @see			GetPropCmd
 * @see         GetAllNamesCmd
 * @see         GetAllPropsCmd
 * @see         PropFindMethod
 * @author		Jonathan Shapiro
 * @version		2.0, 28 Nov 1997
 */
public abstract class PropCmd extends Command
{
	/*Public Interface*/
	public abstract String getHref();

	public abstract Element getOutput();

	public abstract int getInternalStatusCode();

	public static String getStatusString(int internalStatusCode) {
		Integer intKey = new Integer(internalStatusCode);
		Integer intVal = null;

		if (!mapStatusCodes.containsKey(intKey))
		{
			intVal = new Integer(WebDavResponse.SC_INTERNAL_SERVER_ERROR);	
		}
		else
		{
			intVal = (Integer) mapStatusCodes.get(intKey);
		}

		return intVal.toString();
	}


	/* Borrowing Marc's Status Code mapping code */

	/**
	 * This Hashtable contains the mapping of WebDAV subsystem
	 * return codes into the appropriate HTTP return code for
	 * this Method.  This is a static variable.
	 */
	private static Hashtable mapStatusCodes = new Hashtable();

	/**
	 * Adds a new WebDAV -> HTTP status code mapping.  This is a static
	 * method because the mapping is a static variable.
	 * 
	 * @param	nKey - WebDAV status code
	 * @param	nVal - HTTP status code
	 * @see		WebDavRetCode
	 */
	static void addStatusCodeMap(int nKey, int nVal)
	{
		mapStatusCodes.put( new Integer(nKey), new Integer(nVal) );
	}

	/**
	 * Returns the HTTP status code for the WebDav status code specified
	 * by looking it up in the static mapping.  This is a static function.
	 * Since each Method subclass must implement getStatusCode(), the
	 * implementation should simply call the static version of the
	 * function, getStatusCodeImpl().
	 * 
	 * @param	nKey - WebDAV status code
	 */
	public static int getStatusCodeImpl(int nKey)
	{
		Integer intKey = new Integer(nKey);
		Integer intVal = null;

		if (!mapStatusCodes.containsKey(intKey))
		{
			intVal = new Integer(WebDavResponse.SC_INTERNAL_SERVER_ERROR);
		}
		else
		{
			intVal = (Integer) mapStatusCodes.get(intKey);
		}
		return intVal.intValue();
	}

	// Create the mapping of WebDav subsystem return codes to HTTP
	// response codes for the GetMethod.
	static
	{
		addStatusCodeMap(PropertyManager.pm_success,
								WebDavResponse.SC_OK);	

		addStatusCodeMap(PropertyManager.pm_partial_success,
								WebDavResponse.SC_OK);

		addStatusCodeMap(PropertyManager.pm_err_general,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(PropertyManager.pm_err_invalid_input_elem,
								WebDavResponse.SC_BAD_REQUEST);

		addStatusCodeMap(PropertyManager.pm_err_propnotfound,
								WebDavResponse.SC_METHOD_FAILURE);

		addStatusCodeMap(PropertyManager.pm_err_read_only,
								WebDavResponse.SC_NOT_IMPLEMENTED);

		addStatusCodeMap(PropertyManager.pm_err_setprop,
								WebDavResponse.SC_BAD_REQUEST);

		addStatusCodeMap(PropertyManager.pm_err_delete,
								WebDavResponse.SC_METHOD_FAILURE);

	}
};
