// a bug here, file always grows  no good
/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.lockman;

import java.io.*;
import java.util.*;
import webdav.common.*;

/** 
 * This class provide date base routine for lock manager. Purpose of this class to<br>
 * separate db access from the logic of the program  
 * @author Shao Rong
 * @version 1.0
 */
public class LockManDBM implements LockManErrorCode {

  public final static int LOCKMAN_DB_UPDATE = 1;
  public final static int LOCKMAN_DB_DELETE = 2;

  RandomAccessFile d_file;

  LockManDBM () {
    super ();
    d_file = null;
  }

  /** @param dbName : data base name*/
  /** @return int: returned code */
  int db_open (String dbName) {
    try {
      d_file = new RandomAccessFile(dbName, "rw"); 
    }
    catch (IOException e) {
      if (debug) System.out.println ("DB open fail");
      return DB_OPEN_FAIL;
    }
    return LockMan_SUCCESS;
  }

  /** @param dbName : data base name*/
  /** @return int : returned code*/
  int db_close (String dbName) {
    try {
      d_file.close ();
    }
    catch (IOException e) { 
      if (debug) System.out.println ("DB close fail");
      return DB_CLOSE_FAIL;
    }
    return LockMan_SUCCESS;
  }

  /** @param key: the key of the record: uri+lockType+lockScope*/
  /** @param record: the content of the record being filled by db_read*/
  /** @return int : returned code*/
  int db_read (String key, StringBuffer record) {
    try {
      d_file.seek(0);
      String line;
      line = seek (key);
      if (line == null) {
        if (debug) System.out.println ("record not find");
        return DB_REC_NOT_FOUND;
      }
      else {
        record.append (d_file.readLine ());
      }

    }
    catch (IOException e) {

      if (debug) System.out.println ("DB read fail");
      return DB_READ_FAIL;

    }
    return LockMan_SUCCESS;
  }

  /** @param key: the key of the record: uri+lockType+lockScope*/
  /** @param record: the content of the record*/
  /** @param action: action to take*/
  /** @return int : returned code*/
  int db_write (String key, String record, int action) {

    try {
      d_file.seek(0);
      String line ;
      line = seek (key);
      switch (action) {
        case LOCKMAN_DB_UPDATE :
                  if (line == null) {
                    d_file.seek (d_file.length());
                    d_file.writeBytes (key+"\n");
                    d_file.writeBytes (record+"\n");
                  }
                  else {
                    d_file.writeBytes (record+"\n");
                  }
                  break;
        case LOCKMAN_DB_DELETE: 
                  if (line == null) {
                    if (debug) System.out.println (key+" does not exist");
                    return DB_REC_NOT_FOUND;
                  }
                  else {
                     // write to a back up file then write back to the file
                    // just a hack  !!!
                     d_file.seek (0);
                     RandomAccessFile d_file_bk = new RandomAccessFile (".bkfile","rw");
                     try {
                       do {
                         line = d_file.readLine ();
                         while (line.equals (key)) {
                          StringBuffer temp = new StringBuffer ("DELETED");
                          temp.append (key);
                          d_file_bk.writeBytes (temp.toString()+"\n");

                          line = d_file.readLine (); // skip the line with record info
                          temp = new StringBuffer ("DELETED");
                          temp.append (line);
                          d_file_bk.writeBytes (temp.toString()+"\n");

                          line = d_file.readLine ();
                          if (line == null) break;
                         }
                         d_file_bk.writeBytes (line+"\n");
                       } while (line!=null) ;
                
                       // copy back the file
                       d_file.seek (0);
                       d_file_bk.seek (0);
  
                       while ((line=d_file_bk.readLine ()) != null) {
                         d_file.writeBytes (line+"\n");
                       }

                       d_file_bk.close ();
                     }
                     catch (IOException e) {
                       if (debug) System.out.println ("IO exception occurs");
                       return DB_WRITE_FAIL;
                     }
                  }
                  break;
        default : if (debug) System.out.println ("Unkown Action ");
                  return UNKNOWN_ACTION;
      }
    }

    catch (IOException e) {
      if (debug) System.out.println ("DB write failed");
      return DB_WRITE_FAIL;
    }

    return LockMan_SUCCESS;

  }


  /** @param key: the key of the record */
  /** @return null if not found or intenal error, otherwise the line*/
  private String seek (String key) {
    String line=null;
    try {
      do {
        line = d_file.readLine ();
      } while ((line!=null) && !(line.equals(key)));
    }
    catch (IOException e) {
      return null;
    }
    return line; 
  }

}

