/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.nsman;

import webdav.common.Uri;
import webdav.common.UriElementHelper;
import webdav.common.WebdavXML;
import webdav.common.WebDavResponse;
import java.io.File;
import com.ms.xml.om.Element;
import com.ms.xml.om.ElementImpl;

/**
 *
 * @author		Marc Eaddy
 * @version		1.0, 18 Dec 1997
 */
public class DirListItem 
{
	Uri			uri				= null;
	boolean		bIsCollection	= false;
	long		nItemSize		= 0;

	public DirListItem(Uri uri)
	{
		this.uri = new Uri(uri);

		initAttributes(new File(uri.getFullPath()));
	}

	public DirListItem(String str)
	{
		this.uri = new Uri(str);
		initAttributes(new File(str));
	}

	public DirListItem(File file)
	{
		this.uri = new Uri(file);
		initAttributes(file);
	}

	public long getItemSize()
	{
		return nItemSize;
	}

	public boolean isCollection()
	{
		return bIsCollection;
	}

	public Uri uri()
	{
		return uri;
	}

	public String toString()
	{
		return uri.toString() + " Size=" + nItemSize +
			(bIsCollection ? " (COLL)" : "");
	}

	public Element createResponse()
	{
		ElementImpl elemProps =
			new ElementImpl(WebdavXML.ELEM_PROP, Element.ELEMENT);

		Element elemResourceType = createResourceType();

		Element elemHref = UriElementHelper.createHref(uri);

		Element elemStatus = 
			WebDavResponse.createHttpStatusElement(WebDavResponse.SC_OK);

		ElementImpl elemResponse = 
			new ElementImpl(WebdavXML.ELEM_RESPONSE, Element.ELEMENT);

		// Create the prop element
		elemProps.addChild(WebdavXML.elemNewline, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(elemResourceType, null);
		elemProps.addChild(WebdavXML.elemNewline, null);
		elemProps.addChild(WebdavXML.elemTab, null);
		elemProps.addChild(WebdavXML.elemTab, null);

		// Create the response element
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(elemHref,  null);
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(elemProps, null);
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(WebdavXML.elemTab, null);
		elemResponse.addChild(elemStatus, null);
		elemResponse.addChild(WebdavXML.elemNewline, null);
		elemResponse.addChild(WebdavXML.elemTab, null);

		return elemResponse;
	}

	public Element createResourceType()
	{
		ElementImpl elemResourceType =
			new ElementImpl(WebdavXML.ELEM_RESOURCE_TYPE, Element.ELEMENT);

		if (bIsCollection)
		{
			ElementImpl elemCollection =
				new ElementImpl(WebdavXML.ELEM_COLLECTION_RESOURCE_TYPE, Element.ELEMENT);

			elemResourceType.addChild(elemCollection, null);
		}

		return elemResourceType;
	}


	private void initAttributes(File file)
	{
		if (file == null)
		{
			return;
		}

		System.out.println("initAttributes for " + file);

		bIsCollection = file.isDirectory();
		nItemSize = file.length();
	}
};
