/*
Copyright 1997-1998 by Marc Eaddy, Jonathan Shapiro, Shao Rong;
ALL RIGHTS RESERVED
 
Permission to use, copy, modify, and distribute this software and its
documentation for research and educational purpose and without fee is
hereby granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and warranty disclaimer
appear in supporting documentation, and that the names of the
copyright holders or any of their entities not be used in advertising
or publicity pertaining to distribution of the software without
specific, written prior permission.  Use of this software in whole or
in parts for direct commercial advantage requires explicit prior
permission.
 
The copyright holders disclaim all warranties with regard to this
software, including all implied warranties of merchantability and
fitness.  In no event shall the copyright holders be liable for any
special, indirect or consequential damages or any damages whatsoever
resulting from loss of use, data or profits, whether in an action of
contract, negligence or other tortuous action, arising out of or in
connection with the use or performance of this software.
*/

package webdav.propman;

import com.ms.xml.om.*;
import com.ms.xml.util.Name;

/**
 * The LivePropertyHandler interface supports communiction from the 
 * Property Subsystem to a Live Property Handler class. These methods
 * are called in response to a Live Property Handler being registered
 * with a Live Property Delegator
 *
 * @author  Jonathan Shapiro
 * @version 1.0, 11/15/97
 */

public interface LivePropertyHandler {
	/**
	 * Informs the LivePropertyHandler of a default property handler where unhandled requests
	 * can be delegated.
	 * 
	 * @param pm		[IN] An object implementing the PropertyManager interface
	 */
	void setDefaultPropertyManager(PropertyManager pm);

	/**
	 * Construct and return an array of property names that represent the specific properties
	 * handled by this LivePropertyHandler
	 * 
	 * @return		An array of property names
	 */
	Name[] querySupportedProperties();
}
